import { Blueprint, DeepPartial, SchemaValidateOptions } from './types';
export interface OptimalOptions {
    /** Include a filename in validation error messages. Can be used in conjunction with
  `name`. */
    file?: string;
    /** Include a unique identifier in validation error messages. Can be used in conjunction
  with `file`. */
    name?: string;
    /** @internal */
    prefix?: string;
    /** Allow unknown fields to be passed within the object being validated. Otherwise, an error will
  be thrown. */
    unknown?: boolean;
}
export interface Optimal<T extends object> {
    /** Modify optimal options after instantiation. */
    configure: (options: OptimalOptions) => void;
    /**
     * Validate an object with the defined blueprint and options,
     * and return a deeply built object with correct types and values.
     */
    validate: (object: DeepPartial<T>, options?: SchemaValidateOptions) => Required<T>;
}
export declare function optimal<Schemas extends object>(blueprint: Blueprint<Schemas>, baseOpts?: OptimalOptions): Optimal<Schemas>;
//# sourceMappingURL=optimal.d.ts.map